import { useContext, useMemo } from 'react';
import PdeContext from '../contexts/PdeContext.js';
import { getPlatformStrategy } from './common/platformStrategies.js';

/**
 * Hook to use a feature test
 * @param {string} name
 * @param {object} param
 * @param {object} param.attributes
 * @param {function} param.trackExperimentViewed
 * @param {string} param.queryString
 * @param {string} param.adapterId Adapter id to be executed
 * @return {object}
 */
export default function useDecision(name, _temp) {
  var _ref = _temp === void 0 ? {} : _temp,
    attributes = _ref.attributes,
    trackExperimentViewed = _ref.trackExperimentViewed,
    queryString = _ref.queryString,
    adapterId = _ref.adapterId;
  var _useContext = useContext(PdeContext),
    pde = _useContext.pde;
  if (pde === null) {
    throw new Error('[sui-pde: useDecision] sui-pde provider is required to work');
  }
  var data = useMemo(function () {
    try {
      var strategy = getPlatformStrategy({
        customTrackExperimentViewed: trackExperimentViewed
      });
      var forced = strategy.getForcedValue({
        key: name,
        queryString: queryString
      });
      if (forced) {
        if (['on', 'off'].includes(forced)) {
          return {
            enabled: forced === 'on',
            flagKey: name
          };
        }
        return {
          enabled: true,
          flagKey: name,
          variationKey: forced
        };
      }
      var notificationId = pde.addDecideListener({
        onDecide: function onDecide(_ref2) {
          var type = _ref2.type,
            decision = _ref2.decisionInfo;
          var ruleKey = decision.ruleKey,
            variationKey = decision.variationKey,
            decisionEventDispatched = decision.decisionEventDispatched;
          if (type === 'flag' && decisionEventDispatched) {
            strategy.trackExperiment({
              variationName: variationKey,
              experimentName: ruleKey
            });
          }
        }
      });
      var _data = strategy.decide({
        pde: pde,
        name: name,
        attributes: attributes,
        adapterId: adapterId
      });
      pde.removeNotificationListener({
        notificationId: notificationId
      });
      return _data;
    } catch (error) {
      return {
        enabled: false,
        flagKey: name
      };
    }
  }, [trackExperimentViewed, name, queryString, pde, attributes, adapterId]);
  return data;
}